import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)
import java.awt.Color;

/**
 * An edge in a net. An edge connects two nodes. It has a natural length. If it gets 
 * stretched beyond it's natural length, it applies a pulling force on the nodes at 
 * its ends.
 * 
 * @author mik 
 * @version 1.1
 */
public class Edge extends Actor
{
    private static final int STRENGTH = 10; // strength of the pull when stretched
    
    private Node n1;
    private Node n2;
    private double length;
    private Location prevLocN1;
    private Location prevLocN2;
    
    public Edge(Node n1, Node n2, int length) 
    {
        this.n1 = n1;
        this.n2 = n2;
        prevLocN1 = Location.NONE;
        prevLocN2 = Location.NONE;
        this.length = (double)length;
        setImage(new GreenfootImage(100,100));
    }
    
    public void addedToWorld(World world) 
    {
        draw();
        prevLocN1 = n1.getLocation();
        prevLocN1 = n2.getLocation();
    }
    
    /**
     * Act - do whatever the Edge wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        if (hasMoved()) {
            draw();
            prevLocN1 = n1.getLocation();
            prevLocN1 = n2.getLocation();
        }
    }
    
    public void applyForce() 
    {
        Vector force = new Vector(n1.getLocation(), n2.getLocation());
        if ( force.getLength() > length ) {  // if current length > natural length of edge
            force.reduceLength(length);      // current stretch
            force.scale(STRENGTH);
            n1.addForce(force);

            force = force.reverse();
            n2.addForce(force);
        }
    }

    private boolean hasMoved() {
//        return ! ( n1.getLocation().equals(prevLocN1) &&  n2.getLocation().equals(prevLocN2));
        return  Math.abs(n1.getLocation().getX() -  prevLocN1.getX()) > 1 ||
                Math.abs(n1.getLocation().getY() -  prevLocN1.getY()) > 1 ||
                Math.abs(n2.getLocation().getX() -  prevLocN2.getX()) > 1 ||
                Math.abs(n2.getLocation().getY() -  prevLocN2.getY()) > 1;
    }
    
    private void draw() 
    {
        double n1x = n1.getLocation().getX();
        double n1y = n1.getLocation().getY();
        double n2x = n2.getLocation().getX();
        double n2y = n2.getLocation().getY();
        
        int width = (int) Math.abs( n1x - n2x ) + 1;
        int height = (int) Math.abs( n1y - n2y ) + 1;
        int x = (int)(n1x + n2x) / 2;
        int y = (int)(n1y + n2y) / 2;
        setLocation(x, y);
   
        GreenfootImage img = getImage();
        final int imgWidth = img.getWidth();
        final int imgHeight = img.getHeight();
        
        if (imgHeight <= height || imgWidth <= width) {
            img = new GreenfootImage(Math.max(width+1, imgWidth), Math.max(height+1, imgHeight));
            setImage(img);
            //System.out.println("new img " + width + "/" + height);
        }
        else {
            img.clear();
        }

        int lx1 = (int)(x - n1x) + (imgWidth / 2);
        int ly1 = (int)(y - n1y) + (imgHeight / 2);
        int lx2 = (int)(x - n2x) + (imgWidth / 2);
        int ly2 = (int)(y - n2y) + (imgHeight / 2);
        
        img.drawLine(lx1, ly1, lx2, ly2);
    }
}
