import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)

/**
 * A node in a graph.
 * 
 * @author mik
 * @version 1.0
 */
public final class Node extends SmoothMover
{
    private static final int SIZE = 12;
    private static final Vector GRAVITY = new Vector(90, 30);
    
    private static GreenfootImage defaultImage;
    private static GreenfootImage pinnedImage;
    
    private boolean pinned;
    private boolean dragged;

    static {
        defaultImage = new GreenfootImage(SIZE, SIZE);
        defaultImage.setColor(Color.BLACK); 
        defaultImage.fillOval(0, 0, SIZE-1, SIZE-1);
        pinnedImage = new GreenfootImage(SIZE, SIZE);
        pinnedImage.setColor(Color.RED);
        pinnedImage.fillOval(0, 0, SIZE-1, SIZE-1);
    }

    public Node() 
    {
        setImage(defaultImage);
    }
    
    public void pin() 
    {
        pinned = true;
        setImage(pinnedImage);
    }
    
    public void unpin() 
    {
        pinned = false;
        setImage(defaultImage);
    }
    
    /**
     * Act - do whatever the Node wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        if (! doDrag()) {
            if (!pinned) {
                addForce(GRAVITY);
                move();
            }
        }
        if(doubleClick()) {
            if (pinned) {
                unpin();
            }
            else {
                pin();
            }
        }
        stop();
    }
    
    private boolean doubleClick()
    {
        if (Greenfoot.mouseClicked(this)) {
            MouseInfo mouse = Greenfoot.getMouseInfo();
            return (mouse.getClickCount() == 2);
        }
        return false;
    }
    
    private boolean doDrag()
    {
        if (Greenfoot.mouseDragged(this)) {
            MouseInfo mouse = Greenfoot.getMouseInfo();
            setLocation(mouse.getX(), mouse.getY());
            dragged = true;
        }
        if(Greenfoot.mouseDragEnded(this)) {
            dragged = false;
        }
        return dragged;
    }

}
