import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)

/**
 * A thing that can move around with a certain speed.
 * 
 * @author Poul Henriksen
 */
public abstract class SmoothMover extends Actor
{
    private Vector movement = new Vector();
    private Location loc = new Location(0, 0);
    
    public SmoothMover()
    {
    }
    
    /**
     * Create new thing initialised with given speed.
     */
    public SmoothMover(Vector movement)
    {
        this.movement = movement;
    }
    
    /**
     * Move in the current movement direction.
     */
    public void move() 
    {
        loc.move(movement.getX() * 0.02, movement.getY() * 0.02);
        setLocation(loc);
    }
    
    public void setLocation(Location newloc) 
    {
        loc = newloc;
        super.setLocation((int) loc.getX(), (int) loc.getY());
    }
    
    public void setLocation(int x, int y) 
    {
        setLocation(new Location(x, y));
    }

    public Location getLocation() 
    {
        return loc;
    }

    /**
     * Increase the speed with the given vector.
     */
    public void addForce(Vector force) 
    {
        movement.add(force);
    }
    
    /**
     * Increase the speed with the given vector.
     */
    public void stop() 
    {
        movement.setNeutral();
    }
    
    /**
     * Return the current speed.
     */
    public Vector getMovement() 
    {
        return movement;
    }
}
